%**********************************************
%ejemplo de diseo de PID para teora ao 2015
%**********************************************
%   Ejemplo PID 2015
% inicio
clear all
close all
echo off , Tcla= 'Presione una tecla para continuar ...' ; 
PP= 'disp(Tcla), pause' ;    % usar  -->  eval(PP)
clc;
echo on; 

%  Presentacin
%{	
	Planta con retardo.  Controlador tipo PID con Predictor de Smith:
	
    "Controlar el peso de material sobre una cinta transportadora". 
 
   ====>  Descripcin del Sistema:   =========:
   
	La planta a controlar est compuesta de una tolva que vierte material 
	sobre una cinta transportadora. La cantidad de material se regula 
 	mediante una vlvula proporcional. El peso es medido sobre la cinta 
 	con una celda de carga ubicada a una distancia determinada de la tolva. 
 	La velocidad de la cinta es, aproximadamente, constante. Por ello, existe 
 	un retardo, supuesto constante, en la medicin de variaciones del peso 
 	debido a cambios en la vvula de la tolva. 
 	
	Considerando las distintos elementos del conjunto y haciendo algunas 
 	aproximaciones se concluye que un modelo de la planta de cuarto orden 
 	ms el retardo en la salida medida representa bastante bien al sistema.

 	 Se supone que se quiere controlar el sistema con un PID clsico el cual 
 	provee una tensin para abrir o cerrar la vlvula de la tolva en funcin 
	del peso ledo por la celda. 
   ============================================== 
%}
echo off
figure(10)
circuito=imread('ejemploPID.jpg','jpeg');
image(circuito);

%% MODELO ANALGICO de la Planta. (Sin retardo) :      
  s=zpk('s') ;      
   kpa= 0.01 ;    cpa= [-0.35];  ppa= [-0.01; -0.025 ; -0.4; -5];  
   Gpa= zpk(cpa,ppa,kpa); Gpa= ss(Gpa);         % Se usa Mod. de estado para operaciones. 
   disp(' Transferencia de la Planta Continua sin retardo');
   zpk(Gpa)                                    % Se usa Transferencias para presentaciones. 
   
% MODELO DISCRETO de la Planta . (Sin retardo) :   
  Ts= 1;                                        %  perodo de muestreo.  
  Gp= c2d(Gpa,Ts,'zoh');  
  disp(' Transferencia de la Planta Discreta sin retardo');
  zpk(Gp)                                       %  Planta discreta Sin retardo.  (  Gp= GoGp(z) incluye ROC. ) 
  Gp1f=  feedback(Gp,1);  zpk(Gp1f);            %  Discreto: lazo cerrado unitario. (Sin retardo).  
   
%   Agregando el retardo que posee la planta, resulta : 
  Tr= 14.6*Ts;   dd= ceil(Tr/Ts);                  %  dd= Nro. perodos Ts que cubren Tr [s]  
%  z= zpk('z',Ts);  
  Gpr= Gp;  Gpr.OutputDelay= dd;	
  disp(' Transferencia de la Planta Discreta con retardo');
  zpk(Gpr)                                      % Planta discreta con retardo.
 % Gpr1f= feedback(Gpr,1);  zpk(Gpr1f);          % Discreto con retardo: lazo cerrado (unitario).  
 
%%   
% Modelo simulink para simular

% Defino la transferencia del controlador igual a 1

 close all; 
 Gce=tf([1],[1],Ts);    
 [numPID,denPID]=tfdata(Gce,'v');
 [numPIDe,denPIDe]=tfdata(Gce,'v');
 [numPIDy,denPIDy]=tfdata(Gce,'v');
 [numPIDs,denPIDs]=tfdata(Gce,'v');
% Programa SIMULINK para simular el comportamiento del controlador
 EjemploPID2015
 eval(PP)
 % simulacion del modelo con contolador untario.
 sim('EjemploPID2015.mdl');
 figure(1)
 plot(Tiempo,salida_LA);    % respuesta del sistema a lazo abierto con y sin retardo
 title('Respuesta a Escaln.    Discreto. Lazo abierto, Con y Sin Demora.') 
 legend(['Tdemora = ', num2str(Tr),' [s]'],    'location', 'NE' ); 
 grid on
 eval(PP)
 figure(2)
 plot(Tiempo,salida_LC);    % respuesta del sistema a lazo cerrado con retardo
 title('Respuesta a Escaln.  Realimentacin unitaria. Con Demora.') 
 legend(['Tdemora = ', num2str(Tr),' [s]'],    'location', 'NE' ); 
 grid on
%  figure(3)
%  plot(Tiempo,Control);    % respuesta del sistema a lazo cerrado con retardo
%  title('Esfuerzo de control (Sin Controlador).  Realimentacin unitaria. Con Demora.') 
%  legend(['Tdemora = ', num2str(Tr),' [s]'],    'location', 'NE' ); 
%  grid on

%% 
%{ 
  ************************************************************************** 
  Clculo datos para ZIEGLER-NICHOLS usando Planta discreta con demora:
  **************************************************************************
    
%}
clc;
close all;
%  1.-  Clculo Coef. Z-N por respuesta a escaln: 

%  Planta discreta con demora.
%[y1,t1]= step(Gpr) ;          % Respuesta de la planta con demora al escaln
  sim('EjemploPID2015.mdl');
  t1=Tiempo;
  y1=salida_LA;
  
  dy1=y1(2:end)-y1(1:end-1) ;   % Clculo del increlento entre muestras
  imx= find(dy1==max(dy1)) ;  % determinacin del punto de mximo incremento (pendiente mxima)
%{
  	La recta que determina los parmetros es: y= gmax*t + yo, -->  gmax=  dy1(imx)/Ts
    por lo tanto para determinar el corte en cero:  ymx= gmax*(imx*Ts) + yo  --> yo= ymx - gmax*(imx*Ts) 
 	adems,    ymx= y1(imx) ;    y,   0= gmax*R + yo --> R= - yo/gmax
%} 
  gmax= dy1(imx)/Ts;            % pendiente mxima
  yo= y1(imx)- gmax*(imx*Ts);   % ordenada al origen
  L= - yo/gmax                  % punto de corte 

  recta=gmax*Tiempo+yo;         % recta de mxima pendiente
  figure(1)
  plot(t1,y1);
  grid
  hold on
  plot(Tiempo,recta,'r');     % grafico recta de mxima pendiente
  ylim=y1(end);               % valor final de salida
  ulim=Entrada(end);          % valor final de entrada
  Klim=ylim/ulim;             % ganancia
  vfinal=ylim*ones(size(y1)); % recta de valor final
  plot(t1,vfinal,'g');
  hold off;
  xf=find(recta>=ylim);        % cruce de rectas
  tf=xf(1)*Ts;                 % tiempo del cruce de rectas
  R=Klim/(tf-L)                % Pendiente
  
  eval(PP)
  
%  *********** Se usa Z-N a lazo abierto  *********

 % Coef. Z-N  para  PID : 
   kc1= 1.2/(R*L);
   Ti1= 2*L;
   Td1= 0.5*L;                 
 
 % datos para usar en el controlador  
 
 kce= kc1,  Tie= Ti1  ,   Tde= Td1

%  DISEO PID discreto.  ( PID sobre error ) : 
  z= zpk('z',Ts);  
  Ie= (kce*Ts/(2*Tie))*(z+1)/(z-1);      % Transferencia Integral Trapezoidal
  Pe= kce;                               % Transferencia Proporcional
  De= kce*(Tde/Ts)*(z-1)/z;              % Transferencia Derivativa
  Gce= Pe+De+Ie,   Gce= ss(Gce); 
  [numPID,denPID]=tfdata(Gce,'v')
  sim('EjemploPID2015.mdl');
  figure(2)
  grid
  hold
  plot(Tiempo,salida_LC,'r');    % respuesta del sistema a lazo cerrado con controlador y  retardo
%   figure(3)
%   grid
%   hold
%   plot(Tiempo,Control,'r');    % Esfuerzo de Control a lazo cerrado con controlador y  retardo  
%   
%%
%  2 .-  Clculo Coef. Z-N por oscilacin lazo cerrado:   

% Planta discreta con demora:
% Se calcula el margen de ganancia para determinar la ganancia para la oscilacin 
% con la frecuencia de fase 180 se calcula el To
clc;
[mg2,mf2,wg2,wf2]= margin(Gpr);   
kmax= mg2                       % Kmax ganancia para hacer oscilar al sistema a lazo cerrado
To= 2*pi/ wg2                   % periodo de oscilacin

Gpw=d2c(Gpr,'tustin');
Gpw;
figure(3)
w=[0.01:0.0001:.1];
bode(Gpw,w)
grid


  % Coef. Z-N  para  PID : 
 
  kc2= 0.6*kmax;   Ti2= To/2 ;  Td2= To/8 ;

% datos para usar en el controlador  
   
  kce= kc2,  Tie= Ti2  ,   Tde= Td2

%  DISEO PID discreto.  ( PID sobre error ) : 
  z= zpk('z',Ts);  
  Ie= (kce*Ts/(2*Tie))*(z+1)/(z-1);      % Transferencia Integral Trapezoidal
  Pe= kce;                               % Transferencia Proporcional
  De= kce*(Tde/Ts)*(z-1)/z;              % Transferencia Derivativa
  Gce= Pe+De+Ie,   Gce= ss(Gce); 
  [numPID,denPID]=tfdata(Gce,'v')
  sim('EjemploPID2015.mdl');
  figure(2)
  plot(Tiempo,salida_LC,'k');    % respuesta del sistema a lazo cerrado con controlador y  retardo
%   figure(3)
%   plot(Tiempo,Control,'k');    % Esfuerzo de Control a lazo cerrado con controlador y  retardo  
%   
%%

 %{
  	Para resintonizar, se aumentan las ganancias  Td  y  Ti . 
  	Luego de algunas simulaciomnes, queda: 
 %}

kce= 0.85*kc1,  Tie= 2*Ti1  ,   Tde= 1.7*Td1   

%  DISEO PID discreto.  ( PID sobre error ) : 
  z= zpk('z',Ts);  
  Ie= (kce*Ts/(2*Tie))*(z+1)/(z-1);      % Transferencia Integral Trapezoidal
  Pe= kce;                               % Transferencia Proporcional
  De= kce*(Tde/Ts)*(z-1)/z;              % Transferencia Derivativa
  Gce= Pe+De+Ie,   Gce= ss(Gce); 
  [numPID,denPID]=tfdata(Gce,'v')
  sim('EjemploPID2015.mdl');
  figure(2)
  plot(Tiempo,salida_LC,'m');    % respuesta del sistema a lazo cerrado con controlador reajustado y  retardo
%   figure(3)
%   plot(Tiempo,Control,'m');    % Esfuerzo de Control a lazo cerrado con controlador y  retardo  

